﻿using Comfile.ComfilePi;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;

namespace CP_IO13_4C
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            var asm = Assembly.GetExecutingAssembly();
            Cursor = new Cursor(asm.GetManifestResourceStream("CP_IO13_4C.Resources.hiddencursor.cur"));
        }

        Dictionary<uint, Lamp> _lamps;
        Dictionary<uint, Button> _buttons;

        Thread _pollingThread;
        volatile bool _stopThread = false;

        void PollGPIO()
        {
            while (!_stopThread)
            {
                var result = BeginInvoke(new Action(() =>
                {
                    foreach (var input in GPIO.Inputs)
                    {
                        _lamps[input.Number].IsOn = input.IsOn;
                    }

                    foreach (var output in GPIO.Outputs)
                    {
                        output.IsOn = _buttons[output.Number].IsOn;
                    }
                }));

                while (!_stopThread && !result.IsCompleted)
                {
                    Thread.Sleep(1);
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            _serial2StatusLabel.Text = string.Empty;
            _serial3StatusLabel.Text = string.Empty;

            _lamps = new Dictionary<uint, Lamp>
            {
                { GPIO.Input06.Number, lamp1 },
                { GPIO.Input07.Number, lamp2 },
                { GPIO.Input10.Number, lamp3 },
                { GPIO.Input11.Number, lamp4 },
                { GPIO.Input16.Number, lamp5 },
                { GPIO.Input17.Number, lamp6 },
                { GPIO.Input18.Number, lamp7 }
            };

            _buttons = new Dictionary<uint, Button>
            {
                { GPIO.Output19.Number, button1 },
                { GPIO.Output20.Number, button2 },
                { GPIO.Output21.Number, button3 },
                { GPIO.Output22.Number, button4 },
                { GPIO.Output23.Number, button5 },
                { GPIO.Output24.Number, button6 }
            };

            _pollingThread = new Thread(PollGPIO);
            _pollingThread.IsBackground = true;
            _pollingThread.Start();
        }

        private void button12_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            _stopThread = true;
            _pollingThread.Join();
        }

        private void _serial2TestButton_Click(object sender, EventArgs e)
        {
            SerialTest("/dev/serial2", _serial2StatusLabel);
        }

        private void _serial3TestButton_Click(object sender, EventArgs e)
        {
            SerialTest("/dev/serial3", _serial3StatusLabel);
        }

        void SerialTest(string device, Label label)
        {
            label.Text = string.Empty;
            label.ForeColor = Color.White;
            label.Update();

            var baudrates = new int[]
            {
                    9600,
                    19200,
                    38400,
                    57600,
                    115200,
                    230400,
                    460800
            };

            var parities = new List<Parity>();
            parities.AddRange(new Parity[]
            {
                Parity.None,
                Parity.Odd,
                Parity.Even
            });

            try
            {
                using (var port = new SerialPort(device))
                {
                    port.ReadTimeout = 500;
                    port.WriteTimeout = 500;
                    port.ReadBufferSize = 2048;
                    port.WriteBufferSize = 2048;

                    port.Open();
                    port.DiscardInBuffer();
                    port.DiscardOutBuffer();

                    byte[] txBytes = new byte[256];
                    for (int i = 0; i < txBytes.Length; i++)
                    {
                        txBytes[i] = (byte)i;
                    }

                    foreach (var baud in baudrates)
                    {
                        foreach (var parity in parities)
                        {
                            label.Text = $"Testing {baud},{parity}";
                            label.ForeColor = Color.White;
                            label.Update();

                            port.Write(txBytes, 0, txBytes.Length);

                            byte[] rxBytes = new byte[256];
                            int bc = 0;
                            while (bc != txBytes.Length)
                            {
                                bc += port.Read(rxBytes, bc, txBytes.Length - bc);
                            }

                            if (!rxBytes.SequenceEqual(txBytes))
                            {
                                throw new Exception("Data mismatch");
                            }
                        }
                    }
                }

                label.Text = "PASS";
                label.ForeColor = Color.FromArgb(128, 255, 128);
                label.Update();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);

                label.Text = "FAIL";
                label.ForeColor = Color.FromArgb(255, 128, 128);
                label.Update();
            }
        }
    }
}
